#include "msp.h"
#include "communications.h"
#include "common.h"
#include "user_interface.h"
#include "body_temperature.h"
#include "uart.h"
#include "uartBluetooth.h"

static char rx_buffer_uart[MESSAGE_DIMENSION];
static char tx_buffer_uart[MESSAGE_DIMENSION];

static char rx_buffer_bluetooth[MESSAGE_DIMENSION];
static char tx_buffer_bluetooth[MESSAGE_DIMENSION];

void processUARTIncomingPacket(void);
void processBluetoothIncomingPacket(void);
void bluetoothModuleConfiguration(void);
void cleanTXBuffer(char *buffer);

void initializeCommunications(void)
{
    initializeUART();


    initializeUARTBluetooth();

    //Bug - Según el orden de las siguientes, queda mal ingresado la referencia a la función
    setUartBluetoothCallbackFunction(processBluetoothIncomingPacket);

    setUartCallbackFunction(processUARTIncomingPacket);

    if(BLUETOOTH_CONFIGURED == 1)
    {
        P4OUT |= BIT0;
    }
    else
    {
        P4OUT |= BIT0;

        //AT Command Configuration - has to be run only once
        bluetoothModuleConfiguration();
    }
}

void sendPacket(void)
{
    tx_buffer_uart[0] = START_CHARACTER;
    tx_buffer_uart[1] = 3;
    tx_buffer_uart[2] = (char)getLastTemperatureReading();

    tx_buffer_uart[3] = tx_buffer_uart[0] ^ tx_buffer_uart[1] ^ tx_buffer_uart[2];
    tx_buffer_uart[4] = END_CHARACTER;

    //uartTransmit(&tx_buffer_uart[0]);
}

void sendBtPacket(void)
{
    tx_buffer_bluetooth[0] = START_CHARACTER;
    tx_buffer_bluetooth[1] = 3;
    tx_buffer_bluetooth[2] = (char)getLastTemperatureReading();

    tx_buffer_bluetooth[3] = tx_buffer_bluetooth[0] ^ tx_buffer_bluetooth[1] ^ tx_buffer_bluetooth[2];
    tx_buffer_bluetooth[4] = END_CHARACTER;

    uartBluetoothTransmit(&tx_buffer_bluetooth[0]);
}

void processUARTIncomingPacket(void)
{
    uartReceive(&rx_buffer_uart[0]);
    uartBluetoothTransmit(&rx_buffer_uart[0]);
    //turnOnDebugLED();
}

void bluetoothModuleConfiguration(void)
{
    //Not working...

    tx_buffer_bluetooth[0] = 'A';
    tx_buffer_bluetooth[1] = 'T';
    tx_buffer_bluetooth[2] = '+';
    tx_buffer_bluetooth[3] = 'V';
    tx_buffer_bluetooth[4] = 'E';
    tx_buffer_bluetooth[5] = 'R';
    tx_buffer_bluetooth[6] = 'S';
    tx_buffer_bluetooth[7] = 'I';
    tx_buffer_bluetooth[8] = 'O';
    tx_buffer_bluetooth[9] = 'N';
    tx_buffer_bluetooth[10] = '\r';
    tx_buffer_bluetooth[11] = '\n';

    uartBluetoothTransmit(&tx_buffer_bluetooth[0]);
}

void processBluetoothIncomingPacket(void)
{
    uartBluetoothReceive(&rx_buffer_bluetooth[0]);
    //cleanTXBuffer(&tx_buffer_uart[0]);
    //tx_buffer_uart[3] = '\r';
    uartTransmit(&rx_buffer_bluetooth[0]);
}

void cleanTXBuffer(char *buffer)
{
    int i;
    for (i = 0; i<MESSAGE_DIMENSION; i++)
    {
        buffer[i] = 0;
    }
}
